/*
SQL Script for Blog Post
Create example table structures & populate with example data
Create [dbo].[Data] table structure and populate using a cursor/loop
Use [dbo].[Data] to query the results to find which table/columns contain the values you need!
*/
/*
	-- DELETE the example table structures
	drop table [dbo].[Accounts]
	drop table [dbo].[Contacts]
	drop table [dbo].[Transactions]
*/

-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
-- STEP 1:  CREATE example tables with example structure
-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
	create table [dbo].[Accounts]
	(
	account_id int,
	account_name varchar(100),
	account_score varchar(30)
	)

	create table [dbo].[Contacts]
	(
	account_id int,
	contact_id int,
	contact_name varchar(100),
	contact_score varchar(30)
	)

	create table [dbo].[Transactions]
	(
	transaction_id int,
	account_id int,
	contact_id int,
	transaction_date datetime,
	transaction_qty int,
	transaction_amt numeric(18,2)
	)

-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
-- STEP 2:  VIEW empty tables to screen to review the table structures
-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
	select top 100 * from [dbo].[Accounts]
	select top 100 * from [dbo].[Contacts]
	select top 100 * from [dbo].[Transactions]

-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
-- STEP 3:  FIND all TABLES with the COLUMN name like... whatever TEXT you enter in the WHERE clause
-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
	select 
	[TABLE_CATALOG],
	[TABLE_SCHEMA],
	[TABLE_NAME],
	[COLUMN_NAME],
	[DATA_TYPE]
	from [information_schema].[columns]
	where ([COLUMN_NAME] like '%score%')

-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
-- STEP 4:  POPULATE some example values into the table structures...
-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
	insert into [dbo].[Accounts] select 1, 'Account Name 1', 'GOLD'
	insert into [dbo].[Accounts] select 2, 'Account Name 2', 'SILVER'
	insert into [dbo].[Accounts] select 3, 'Account Name 3', 'BRONZE'
	insert into [dbo].[Accounts] select 4, 'Account Name 4', 'GOLD'
	insert into [dbo].[Accounts] select 5, 'Account Name 5', 'SILVER'

	insert into [dbo].[Contacts] select 1, 1, 'Contact Name 1-1', 'GREEN'
	insert into [dbo].[Contacts] select 1, 2, 'Contact Name 1-2', 'GREEN'
	insert into [dbo].[Contacts] select 2, 1, 'Contact Name 2-1', 'RED'
	insert into [dbo].[Contacts] select 2, 2, 'Contact Name 2-2', 'RED'
	insert into [dbo].[Contacts] select 5, 1, 'Contact Name 5-1', 'RED'

-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
-- STEP 5:  VIEW populated tables to screen to review the data values
-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
	select top 100 * from [dbo].[Accounts]
	select top 100 * from [dbo].[Contacts]
	select top 100 * from [dbo].[Transactions]

-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
-- STEP 6:  CREATE the Structure used to capture table_name, column_name, and values
-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
	create table [dbo].[Data]
	(
	table_name varchar(100),
	column_name varchar(100),
	column_value varchar(100)
	)

-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>	
-- STEP 7:  GET list of the tables on the Database & loop through each table/column to load the data
-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
	-- DECLARE Variables
	declare @select varchar(max)
	declare @table_name varchar(max)
	declare @column_name varchar(max)

	-- DECLARE cursor
	declare db_cursor cursor for
	select
	[TABLE_NAME],
	[COLUMN_NAME]
	from [information_schema].[columns]
	where ([TABLE_NAME] <> 'Data')
	group by [TABLE_NAME],
	[COLUMN_NAME]

	-- OPEN cursor and get the first set of values
	open db_cursor  
	fetch next from db_cursor into @table_name, @column_name

	-- LOOP through each row of the cursor
	while @@FETCH_STATUS = 0  
	begin  

		-- BUILD the dynamic query statement
		set @select = 
		'select ''' + @table_name + '''' + ' as table_name' + ', ' + '''' + @column_name + '''' + ' as column_name ' + ', [' + @column_name
		+ '] as column_value into [dbo].[temp] from [dbo].[' + @table_name + '] group by [' + @column_name + ']'

		-- EXECUTE the dynamic query statement
		exec (@select)
		
		-- POPULATE the table
		insert into [dbo].[Data] select * from [dbo].[temp]
		drop table [dbo].[temp]
	
		-- GET the next set of values
		fetch next from db_cursor into @table_name, @column_name
	
	end 

	-- CLOSE and DROP the cursor
	close db_cursor  
	deallocate db_cursor 

-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
-- STEP 8:  VIEW results to screen
-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
	select * from [dbo].[Data]

-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
-- STEP 9:  Query the [dbo].[Data] table to find the name of the column and the table that has the "value" you need
-- <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><> <><><><><>
	select * from [dbo].[Data] where ([column_value] IN ('GOLD','SILVER','BRONZE'))




	/*
	truncate table [dbo].[Data]

	-- Manually POPULATE values into the table (this is what is created by STEP 7)
	insert into [dbo].[Data] select 'Accounts', 'account_id', '1'
	insert into [dbo].[Data] select 'Accounts', 'account_id', '2'
	insert into [dbo].[Data] select 'Accounts', 'account_id', '3'
	insert into [dbo].[Data] select 'Accounts', 'account_id', '4'
	insert into [dbo].[Data] select 'Accounts', 'account_id', '5'

	insert into [dbo].[Data] select 'Accounts', 'account_name', 'Account Name 1'
	insert into [dbo].[Data] select 'Accounts', 'account_name', 'Account Name 2'
	insert into [dbo].[Data] select 'Accounts', 'account_name', 'Account Name 3'
	insert into [dbo].[Data] select 'Accounts', 'account_name', 'Account Name 4'
	insert into [dbo].[Data] select 'Accounts', 'account_name', 'Account Name 5'

	insert into [dbo].[Data] select 'Accounts', 'account_score', 'GOLD'
	insert into [dbo].[Data] select 'Accounts', 'account_score', 'SILVER'
	insert into [dbo].[Data] select 'Accounts', 'account_score', 'BRONZE'

	select * from [dbo].[Data]
	*/